databaseView <-
function(con, callbacks = NULL, databases = getDatabases(con)[,1], lazy = TRUE)
{
  # User can specify the subset of databases of interest.
  #
  # callbacks
  #
  #
  # lazy is not yet working but is intended to create a version of the tree
  # in which we expanding a node will cause the children to be computed
  # on-demand and not ahead of time. 

 require(RGtk)
  
 win <- gtkWindow()

 h <- gtkVPaned()


 itemCallbacks <- list(NULL,
                        function(item, ancestors) {
                          item$AddCallback("enter-notify-event",
                                             function(tb, w, cross) {
                                               n <- quickSQL(con, paste("select count(*) from", paste(tb, collapse=".")))
                                               status$Push(statusContext,paste("# records", n))
                                             }, data = ancestors)
                        },
                          function(item, ancestors) {
                            item$AddCallback("enter-notify-event",
                                             function(tb, w, cross) {
                                               fields <- getTableFields(con, tb[2], tb[1])
                                               tp <- fields[fields[,1] == tb[3], "Type"]
                                               status$Push(statusContext,paste("type:", as.character(tp)[1]))
                                             }, data = ancestors)
                          }
                        )
 
 status <- gtkStatusbar()
 statusContext <- status$GetContextId("description")
 
 sw <- gtkScrolledWindow()
 f <- if(lazy) databaseLazyTreeView else databaseTreeView
 tr <- f(con, callbacks = callbacks, databases = databases, itemHandlers = itemCallbacks)
 sw$AddWithViewport(tr)
 sw$SetUsize(200, 360)
   # Now put the two top-level widgets into the window's container
 h$Add(sw)
 h$Add(status)

 win$SetUsize(200, 400)
 win$Add(h)
 win$Show()

 invisible(win)
}  

databaseTreeView <-
function(con, callbacks = NULL, top = gtkTree(), databases = getDatabaes(con)[,1], itemHandlers = NULL)
{
  info <- getInfo(con)
  topItem <- gtkTreeItem(paste(info$user, info$host, sep="@"))
  top$Append(topItem)

  tree <- createDatabaseTree(databases, callbacks = callbacks,
                       children = list(function(x) {
                                        getTables(con, x[1])
                                       },
                                       function(x) {
                                         getTableFields(con, x[2], x[1])[,1]
                                       }), itemHandlers = itemHandlers)

  topItem$SetSubtree(tree)

  top
}

databaseLazyTreeView <-
function(con, callbacks = NULL, top = gtkTree(), databases = getDatabases(con)[,1], itemHandlers = NULL)
{
  info <- getInfo(con)
  topItem <- gtkTreeItem(paste(info$user, info$host, sep="@"))
  top$Append(topItem)

  tree <- gtkTree()
  for(i in databases) {
    item <- gtkTreeItem(i)
    tree$Append(item)

    if(!is.null(itemHandlers[[1]]))
      itemHandlers[[1]](item, i)
    
    tmp <- gtkTree()
    item$SetSubtree(tmp)

    item$AddCallback("expand", function(x, w) {
                                        tr <- x[[2]]
                                        db <- x[[1]]
                                        if(length(gtkChildren(tr)) > 0)
                                          return()
                                        
                                        tables <- getTables(con, db)
                                        for(i in tables) {
                                          it <- gtkTreeItem(i)
                                          tr$Append(it)
                                          if(!is.null(itemHandlers[[2]]))
                                            itemHandlers[[2]](it, c(db, i))

                                          ftree <- gtkTree()
                                          it$SetSubtree(ftree)


                                          it$AddCallback("expand",
                                                          function(x, w) {
                                                            tr <- x[[1]]
                                                            if(length(gtkChildren(tr)))
                                                              return()

                                                            fields <- getTableFields(con, x[[2]], x[[3]])

                                                            for(i in fields[,1]) {
                                                              it <- gtkTreeItem(i)
                                                              tr$Append(it)
                                                              if(!is.null(itemHandlers[[3]]))
                                                                itemHandlers[[3]](it, c(x[[3]], x[[2]], i))
                                                            }
                                                          }, data = list(ftree, i, db))
                                        }
                                      }, data = list(i, tmp))
  }
  
  topItem$SetSubtree(tree)

  top
}

createDatabaseTree <-
function(x, callbacks = NULL, children = NULL, ancestors = character(), expands = NULL, itemHandlers = NULL)
{
  if(length(x) == 0)
    return(NULL)
  
  tree <- gtkTree()

  kidFuns <- if(length(children) > 1)  children[-1] else NULL
  kidExpands <- if(length(expands) > 1) expands[-1] else NULL  
  itemFuns <- if(length(itemHandlers) > 1) itemHandlers[-1] else NULL

  for(i in x) {
    item <- gtkTreeItem(i)
    tree$Append(item)

    if(length(itemHandlers) && !is.null(itemHandlers[[1]])) 
      itemHandlers[[1]](item, c(ancestors, i))

    if(length(expands)) {
      item$AddCallback("expand", expands[[1]], c(ancestors, i))
    } else if(length(children) > 0) {
      tmp <- createDatabaseTree(children[[1]](c(ancestors, i)), callbacks = callbacks, children = kidFuns, ancestors = c(ancestors, i), itemHandlers = itemFuns)
      if(!is.null(tmp))
        item$SetSubtree(tmp)
    }
  }

  tree
}
  
