#
# This is a viewer for color schemes as currently defined in 
# GGobi. See there for more details. The hope is that these
# or something containing the same basic information can become
# a standard way to 

colorSchemesView <-
function(schemes, schemeCallback = NULL)
{
  # A GUI for viewing (GGobi) colorschemes.
  # 
  
  require(RGtk)
  require(gtkDevice) # Fix this.
  draw <- gtkDrawingArea()
  draw$SetUsize(300, 400)  
  asGtkDevice(draw)
  dev <- dev.cur()

  schemeCallback <- function(scheme, w) {
    if(!is.null(scheme[[1]]$description)) {
       text$LoadFromString(scheme[[1]]$description, -1)
     }

    dev.set(dev)
    displayScheme(scheme[[1]], scheme[[2]])       
  }
  
  win <- gtkWindow(show = FALSE)
  h <- gtkHPaned()

  book <- gtkNotebook()
  
  tr <- colorSchemesTree(schemes, schemeCallback = schemeCallback)
  sw <- gtkScrolledWindow()
  sw$AddWithViewport(tr)
  book$AppendPage(sw, gtkLabel("By type"))

 tr <- colorSchemesBySize(schemes, schemeCallback = schemeCallback)
 sw <- gtkScrolledWindow()
 sw$AddWithViewport(tr)
 book$AppendPage(sw, gtkLabel("By size"))

  book$SetUsize(250, 600)
  
  h$Add(book)

  v <- gtkVPaned()
  v$Add(draw)

  fr <- gtkFrame("Description")
  text <- gtkHTML()
  sw <- gtkScrolledWindow()
  sw$Add(text)
  fr$Add(sw)
  v$Add(fr)
  h$Add(v)


  win$Add(h)
  win$SetUsize(750, 500)
  win$Show()

  list(win = win, tree = tr)
}  

colorSchemesBySize <-
function(schemes, top = gtkTree(), schemeCallback = NULL)
{
 colorSchemesTree(schemes, schemeCallback = schemeCallback,
                   top = top,
                   group = function(x) length(x$colors),
                   groupId = function(x, y) length(x$colors) == y)
}


colorSchemesTree <-
function(schemes, top = gtkTree(), schemeCallback = NULL, group = class, groupId = function(x, y) class(x)[1] == y)
{
  # Create a tree for the given list of color schemes.
  # These are grouped by type/class.

   # get the different classes of schemes that we have.
  types <- unique(sapply(schemes, group))

   # top node.
  item <- gtkTreeItem("Color Schemes")
  top$Append(item)

    # the sub-tree for the different types.
  tr <- gtkTree()
  item$SetSubtree(tr)

   # loop over the different 
  for(i in types) {
      # find the schemes for this type.
    which <- sapply(schemes, groupId, i)

      # if there are none, skip this type!
      # This means we miss "Virtual" classes.
    if(sum(which) == 0)
      next

      # create an item in the types tree for this type.
    it <- gtkTreeItem(i)
    tr$Append(it)

      # Now create a sub-tree for the different schemes in this type.
    tmp <- gtkTree()
    it$SetSubtree(tmp)
    for(s in names(schemes)[which]) {
       # Each scheme.
      it <- gtkTreeItem(s)
      tmp$Append(it)
      if(!is.null(schemeCallback))
        it$AddCallback("select", schemeCallback, list(schemes[[s]], s))
      it$SetSubtree(colorSchemeTree(schemes[[s]]))
    }
  }
 
  top
}

colorSchemeTree <-
function(scheme, tree = gtkTree())
{
  # Given a scheme, show its colors.
  # Add the background and accents.
 for(i in names(scheme$colors)) {
   it <- gtkTreeItem(i)
   tree$Append(it)
 }

 tree
}  

displayScheme <-
function(scheme, name)
{
  ## Draw a collection of rectangles, one for each color in the scheme.
  ## Super-impose these on a rectangle painted in the background for the
  ## scheme. And label each rectangle with its color name using the color
  ## of the annotations to draw the text.
  
  n <- length(scheme$colors)
  plot(0:n, seq(0, 1.5, length = n+1), type="n", axes=FALSE, xlab="", ylab="")
  title(main = name, cex = 1.4)

    # Compute these from the scheme.
  bg <- scheme$background$value
  annotations <- scheme$annotations$value
  
  x <- seq(0, n-1)
  rect(0,.1, n, 1.5, col= bg)
  rect(x, rep(.75, n), x+1, rep(1.25, 4),
        col = sapply(scheme$colors, function(x) x$value))
  text(x + .5, .75, names(scheme$colors), adj = c(1,1), srt=45,
         col = annotations)
}  

if(FALSE) {
  # Not used
 colorValues <-
  function(s) {
  # Get the values for the colors in the scheme which can be passed
  # to high-level plot commands.
  
  sapply(s$colors, function(x) x$value)
 }
}
