getRPackageInfo <-
function(mirrors = c(CRAN = contrib.url(CRAN),
#                     Devel = paste(CRAN, "/src/contrib/Devel", sep=""),
                     Omegahat = "http://www.omegahat.org/download/R/packages",
                     BioConductor = "http://www.bioconductor.org/packages/release/distrib",
#                     Old = paste(CRAN, "/src/contrib/Old", sep="")
                    ),
         CRAN=getOption("CRAN"), full = FALSE, ...)
{
 pkgs <- list()
 if(full) {
   tmpFile <- tempfile()
   on.exit(unlink(tmpFile))
 }

 for(i in names(mirrors)) {
   if(full) {
     download.file(paste(mirrors[i], "PACKAGES", sep="/"), destfile = tmpFile, ...)
     pkgs[[i]] <- read.dcf(tmpFile)
   } else
     pkgs[[i]] <- CRAN.packages(contriburl = mirrors[i])

#   pkgs[[i]] <- na.omit(pkgs[[i]])
 }

 invisible(pkgs)
}

CRANView <-
function(mirrors = c(CRAN = contrib.url(CRAN),
#                     Devel = paste(CRAN, "/src/contrib/Devel", sep=""),
                     Omegahat = "http://www.omegahat.org/download/R/packages",
                     BioConductor = "http://www.bioconductor.org/packages/release/distrib",
#                     Old = paste(CRAN, "/src/contrib/Old", sep="")
           ),
         CRAN=getOption("CRAN"),
         pkgs = getRPackageInfo(mirrors, full = TRUE))
{
 library(RGtk)
 library(RGtkHTML)
 
 top <- gtkTree()
 item <- gtkTreeItem("R packages")

 activePackage <- character(2)
 
 mirrorCB <-
   function(name, w) {

     descs <- paste(paste("<tr align='left'><td>",
                           paste(
                                  paste("<a href='", paste(name, pkgs[[name]][,"Package"], sep=":"), "'>", sep=""),
                                  paste(pkgs[[name]][,"Package"], "</a>", sep=""),
                               "</td>"),
                           paste("<td>", pkgs[[name]][,"Description"],"</td></tr>"),
                         collapse="\n"))            
     
      txt <-
        paste("<h3>", name, "</h3>",
          "<table>",
          "<tr><td>URL</td><td>", mirrors[name], "</td></tr>",
          "<tr><td>Number of packages</td><th>", nrow(pkgs[[name]]), "</th></tr>",          
          "</table>",
          "<hr width='50%'>",
          "<table>",
             descs ,
           "</table>",
         sep="\n")
     
      html$LoadFromString(txt, -1)
   }

   showPackage <-
     function(name, packageName, package = NULL) {

       if(is.null(package)) {
         package <- pkgs[[packageName]]
       }

       activePackage <<- c(name, packageName)
       
       info <- package[package[, "Package"] == name,]

       form <- paste("<form><input type='submit' name='Install' value='Install'></form>")
       
       which <- (1:length(info))[info != "NA"]
       fields <- paste(
                       paste("<tr align='left'><th>", colnames(package)[which], "</th>"),
                       paste("<td>", info[which], "</td></tr>"),
                       collapse= "\n", sep=""
                      )

       txt <- paste("<h2>", packageName, "</h2>",
                    "<table>",
                    fields,
                    "</table>",
                    form,
                    collapse="\n"
                   )
       
       html$LoadFromString(txt, -1)
     }

 for(i in names(pkgs)) {
   if(is.null(pkgs[[i]]) || nrow(pkgs[[i]]) == 0)
     next
   it <- gtkTreeItem(i)
   tr <- createPackageTree(pkgs[[i]], callback = function(info, ...) { showPackage(info$packageName,
                                                                                   info$mirror)}, mirror = i)
   top$Append(it)

   it$SetSubtree(tr)

   it$AddCallback("select", mirrorCB, i)
 }

 win <- gtkWindow()
 win$SetTitle("CRAN Package Viewer")
 
 h <- gtkHPaned()
 
 sw <- gtkScrolledWindow()
 sw$AddWithViewport(top)
 h$Add(sw)

 sw <- gtkScrolledWindow()
 html <- gtkHTML()
 sw$Add(html)
 h$Add(sw)

 html$AddCallback("link-clicked", function(h, link, ...) {
                                     els <- strsplit(link, ":")[[1]]
                                     showPackage(els[2], els[1])
                                  })
 html$AddCallback("submit", function(...) {
     # have to override install.packages() and do it ourselves
     # as that doesn't play nice with asking to delete file,
     # put message in the status bar about intermediate steps, etc.
                                   install.packages(activePackage[1], contriburl = mirrors[activePackage[2]])
                                  }) 
 
 win$Add(h)
 win$SetUsize(400, 550)
 win$Show()

 invisible(list(window = win, packages = pkgs))
}

createPackageTree <-
function(pkgs = CRAN.packages(), top = gtkTree(), callback = NULL, mirror = NULL)
{
  for(i in 1:nrow(pkgs)) {
    it <- gtkTreeItem(pkgs[i,"Package"])
    top$Append(it)
    if(!is.null(callback))
      it$AddCallback("select", callback, list(packageName = pkgs[i,"Package"],
                                              mirror = mirror))
  }

  top
}  
