#
# Display the collection of definitions within a directory of S language
# files within a GUI to allow the user explore what is there.
# Display the files as top-nodes in a tree each of whose sub-nodes
# are the names of the top-level defined objects.
#
# Can also show an inverted view which presents the definitions
# as a list and the associated files.
# If the files were part of a package, that would be another story!
#

sourceManager <-
function(dir="R", ...)  
{
  require(FileManager)
  require(RGtk)
  
  d <- functionDirectory(dir)

  win <- gtkWindow(show = FALSE)

  showFun <- function(name, file) {

        # Use RSXMLObjects to turn this object into XML/HTML
        # with proper markup for the different language elements,
        # links to different variables, objects, etc.
                                 source(name[["file"]])
                                 def <- textConnection(".def","w")
                                 sink(def)
                                 print(get(name[["name"]]))                                 
                                 sink()
                                 close(def)
                                 txt <- paste("<html><body><pre>\n", paste(get(".def"), collapse="\n"), "\n</pre>></body></html")
                                 defDisplay$LoadFromString(txt, -1)
                                }
  
  tr <- gtkTree()
  for(i in names(d$defs)) {
    el <- gtkTreeItem(i)
    tr$Append(el)
    if(length(d$defs[[i]]) > 0) {
      subTr <- gtkTree()      
      for(j in names(d$defs[[i]])) {
        tmp <- gtkTreeItem(j)
        subTr$Append(tmp)
        tmp$AddCallback("select", showFun, data = c(file = i, name=j), object = TRUE)
      }
      el$SetSubtree(subTr)
    }
  }

 
  h <- gtkHPaned()


  book <- gtkNotebook()

  sw <- gtkScrolledWindow()
  sw$AddWithViewport(tr)
  sw$SetUsize(200, 500)

  book$AppendPage(sw, gtkLabel("Files"))

  funList <- gtkList()
  tmp <- d$fileNames[order(names(d$fileNames))]
  for(i in names(tmp)) {
    item <- gtkListItem(i)
    funList$Add(item)
    item$AddCallback("select", showFun, data = c(file = d$fileNames[i], name = i))
    item$AddCallback("enter-notify-event",
                      function(fileName, ...) {
                        status$Push(statusContext, fileName)
                      }, d$fileNames[i])
  }
  box <- gtkVBox(FALSE, 3)
  sw <- gtkScrolledWindow()
  sw$AddWithViewport(funList)
  box$PackStart(sw, expand = TRUE)

  status <- gtkStatusbar()
  statusContext <- status$GetContextId("description")
  box$PackEnd(status, expand = FALSE)

  book$AppendPage(box, gtkLabel("Functions"))
  
  h$Add(book)

  defDisplay <- gtkHTML()    
  sw <- gtkScrolledWindow()
  sw$Add(defDisplay)
  h$Add(sw)  
  
  win$Add(h)

  win$SetUsize(600, 500)
  win$Show()

  invisible(win)
}  
