\name{addMIMEObjectHandler}
\alias{addMIMEObjectHandler}
\title{Register a handler function for a type of OBJECT tag in an HTML document.}
\description{

  This allows one to register a handler function
  that is responsible for processing \code{OBJECT} nodes in an HTML
  document that have the specified \code{type}.
  This function is called with the \code{GtkHTML} widget
  and the \code{HTMLEmbedded} widgets as arguments
  and it is responsible for processing
  the contents of the \code{OBJECT},
  adding any GUI component to the \code{HTMLEmbedded} object,
  and returning an S object that might be used by other
  plugins within the document.
}
\usage{
addMIMEObjectHandler(name, handler)
}
\arguments{
  \item{name}{a string giving the exact name of the MIME type as it is
    used in the type attribute of an \code{OBJECT} node.
    }
    \item{handler}{the S function that is to be used to process an
      \code{OBJECT} node with this MIME type. This is a function that
      should accept 3 arguments: the GtkHTML widget in which the
      processing is occurring, the
      GtkHTMLEmbedded object being processed, and
      an optional 3rd argument which is the environment
      in which to perform any evaluations, etc. when processing
      the object. The default for this third argument should be
      \code{\link{globalenv}}.
      }
}
\value{
 The updated list  of MIME type handlers.
}

\references{
  \url{http://www.omegahat.org/RGtk},
  \url{http://www.gtk.org}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
  \code{\link{viewHtml}}
  \code{\link{embeddedObjectHandler}}
  \code{\link[IDocs]{docView}}  
}
\examples{
 addMIMEObjectHandler("app/x-R-histogram",
                       function(w, obj, envir = globalenv()) {
                          w = gtkDrawingArea()
                          asGtkDevice(w)
                          obj$Add(w)

                          tmp = obj$GetParameter("data")
                          v = eval(parse(text = tmp), envir = envir)
                          if(is.vector(v))
                            hist(v)
                          dev.cur()
                       })
}
\keyword{interface}
\keyword{programming}

