\name{databaseView}
\alias{databaseView}
\title{GUI for displaying database, tables and schema}
\description{
  This provides a GUI for browsing the structure
  of a database server. Specifically, it
  creates a \code{GtkTree} object
  which shows the hierarchical content of the
  server, i.e.   databases, tables within databases,
  fields within tables as an expandable and interactive tree.
  One can customize the behavior of the tree  by providing one or more
  callbacks for the different nodes.
}
\usage{
databaseView(con, callbacks = NULL, databases = getDatabases(con)[, 1], lazy = FALSE)
}
\arguments{
  \item{con}{the RSDBI connection object to the server}
  \item{callbacks}{a list of callbacks which are to be registered
    with the the tree items at successive levels of the tree.
    The first function in the list is used for database name items.
    The second element is used for tables within those databases.
    And the final element is used for fields within those tables.
    Any of them can be \code{NULL}.}
  \item{databases}{a character vector giving the names of the databases
    to show. This defaults to all of the databases in the server.
    One can specify a subset of the entire list for efficiency and focus.}
  \item{lazy}{a logical value, which if TRUE specifies that the
    sub-nodes should not be created until they are expanded.
    Otherwise (\code{FALSE}), the entire contents of the tree
    are queried from the database before it is displayed.
    This trades off delays in initialization when querying the entire
    hierarcy of the database server for more uniform delays when
    examining the tree.}
}
\value{
  The top-level Gtk GUI window displaying the class hierarchy.
  This is an object of class   \code{GtkWindow}.
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}
\references{
  \url{http://www.omegahat.org/RGtk},
  \url{http://www.omegahat.org/RSDBI}
}


\seealso{
  \code{\link{createDatabaseTree}}
  \code{\link{createDatabaseTree}}
  \code{\link{databaseLazyTreeView}}  
}

\examples{
\dontrun{
 library(RMySQL)
 m <- dbManager("MySQL")
 con <- dbConnect(m)
 databaseView()
}
}
\keyword{GUI}
\keyword{database}
