\name{embeddedObjectHandler}
\alias{embeddedObjectHandler}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Process OBJECT tags in HTML viewer}
\description{

  This is a callback or signal handler
  that can be  used for processing \code{<OBJECT>}
  tags in HTML when rendering a document
  with the \code{GtkHTML} widget.
  When the HTML parser encounters each <OBJECT> tag,
  it calls the \code{object_requested} callback.
  This callback is expected to process the \code{<OBJECT>}
  instruction. It can create widgets or perform other operations
  that have desired side-effects.
  This function is an extensible mechanism for handling
  different MIME types that are encountered in the
  \code{<OBJECT>} tag.
  One can add handler functions for particular MIME types
  by adding them to the \code{.HTMLObjectTypeHandlers}
  list, using the name of the MIME type as the index.
   
}
\usage{
embeddedObjectHandler(html, obj, envir = globalenv(), asIs = FALSE)
}
\arguments{
  \item{html}{the \code{GtkHTML} widget in which the
    \code{<OBJECT>} is to be rendered}
  \item{obj}{information from
    the \code{<OBJECT>} tag
    that describes what is being requested.
    This is an object
    of class \code{gtkHTMLEmbedded}
    and it has information about the
    type of object to be created,
    the parameters/attributes in the \code{<OBJECT>},
    etc.
    One can access these via the properties
    (using the \code{[[} operator or \code{\link[RGtk]{gtkObjectGetArgs}})
    and the
    \code{gtkHTMLEmbedded} methods (e.g. \code{\link[RGtk]{gtkHTMLEmbeddedGetParameter}}).
  }
  \item{envir}{the environment in which to evaluate the code associated
    with the \code{OBJECT} description. This is passed to the
    MIME handler functions so that they can create the correct
    environments for functions defined within the processing of the
    OBJECT, find other variables, etc.}
  \item{asIs}{if \code{TRUE} return the value directly from
    the MIME handler function. This is useful if this function is being
    called by a wrapper handler. Otherwise if it is being called
    directly by the HTML widget, it should return a logical value.
  }
}
\details{
  This function is registered with a \code{gtkHTML}
  object and is invoked when each time
  the renderer processes a \code{<OBJECT>} tag.
}
\value{
  This function attempts to create
  a suitable object. If it is a widget,
  the function inserts it into the 
}
\references{
  \url{http://www.omegahat.org/RGtk},
  \url{http://www.gtk.org}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{viewHtml}}
}

\examples{
 viewHtml(system.file("examples", "form.html", package="RGtkViewers"))

  # load an R library given a node of the form
  #  <OBJECT type="app/x-R-library" name="mva" />
 .HTMLObjectTypeHandlers[["app/x-R-library"]] = function(html, obj) {
      library(obj[["name"]])
  }
}
\keyword{GUI}

