\name{xmlTreeView}
\alias{xmlTreeView}
\title{Display a GUI tree representing an XML hierarchy}
\description{
  This displays a top-level window containing a
  scrolling \code{GtkTree} object that represents
  the hierarchy of an XML document or node.
}
\usage{
xmlTreeView(doc, createItem = function(x) gtkTreeItem(xmlName(x)))
}
\arguments{
  \item{doc}{the source of the XML nodes. This can be the name of a file
    or URI in which case that document is read,
    or it can be the result from an earlier call to 
    \code{\link[XML]{xmlTreeParse}.}
    or \code{\link[XML]{xmlRoot}}, i.e. an \code{XMLDocument} \code{XMLNode} object.}
  \item{createItem}{a function that is used to create the
    \code{GtkTreeItem} from the \code{XMLNode} object.
    This makes it easy to create different views of the
    nodes and to specify any callbacks to customize the behavior of
    the nodes in the tree.
    See \code{\link{xmlView}} which is the function that uses this.
  }
}
\value{
  An list containing the
  \code{GtkWindow} and the \code{GtkTree}
  objects.
}

\author{Duncan Temple Lang <duncan@research.bell-labs.com>}
\references{
  \url{http://www.omegahat.org/RGtk}
  \url{http://www.omegahat.org/RSXML}    
}

\seealso{
  \code{\link{xmlView}}
  \code{\link[XML]{xmlTreeParse}}  
}

\examples{
   # basic view that uses the xmlName() value of a node.
  xmlTreeView("http://www.ggobi.org/data/flea.xml")

   # customized view that uses the xmlName() and class of the node
  xmlTreeView("http://www.ggobi.org/data/flea.xml",
               createItem = function(x)
                               gtkTreeItem(paste(xmlName(x), class(x)[1], sep=":"))) 


   # basic view that uses the xmlName() value of a node.
  xmlTreeView("http://www.ggobi.org/data/flea.xml",
               createItem = function(x) {
                              it <- gtkTreeItem(xmlName(x))
                              if(inherits(x, "XMLTextNode"))
                                it$AddCallback("select",
                                                function(node, w) {
                                                  print(xmlValue(node))
                                                }, data = x)
                              it
                            })
}
\keyword{GUI}
